clear all; clc;

%% Extract data from the txt file into matlab workspace and define known parameters
data_sys=load('logfile1.txt');
K_m=5;
del_t=0.02;

%% Selecting the estimation and validation dataset
L=length(data_sys);
F=0.7;%First fraction (portion) of logfile to be used for model adaption:
N=floor(L*F); %N is number of samples used for estimation.

%Rest used for visual validation.
t_estim=del_t*[1:N]';
u_estim=data_sys(1:N,2);
y_estim=data_sys(1:N,3);
t_valid=del_t*[N+1:L]';
u_valid=data_sys(N+1:L,2);
y_valid=data_sys(N+1:L,3);
t_total=del_t*[1:L]';
u_total=data_sys(1:L,2);
y_total=data_sys(1:L,3);

%% Creating the system 
C = u_estim(1:end-1,:);      %control signal
S = y_estim(1:end-1,:)./K_m; %speed in the previous time step
Y = y_estim(2:end,:)./K_m;   %speed 
phi=[S C ones(length(C),1)];
theta = inv(phi'*phi)*phi'*Y;

%% Calculating the parameters
T=del_t/(1-theta(1))
K=theta(2)*T/(del_t)
L=theta(3)*T/(K*del_t)

%% Validating the model
C = u_total(1:end-1,:);      %control signal
S = y_total(1:end-1,:)./K_m; %speed in the previous time step
Y = y_total(2:end,:)./K_m;   %speed in the current time step

y_est=[S C ones(length(C),1)]*theta;

%% Plot and data Visualisation
plot(t_total(2:end),y_est,'-r',t_total(2:end),Y,'b--o');
xlabel('Time (s)');
ylabel('Measurement');
legend('Estimated Value','Real Value');