%Finn Haugen (finn@techteach.no)
%24.2 2018
%----------------------------------------------------------
disp('-------------------')
disp('Grid search method to solve an')
disp('optimization problem:')
disp('min_x f(x)')
disp('s.t.')
disp('model: f= function which calculates the error "AC_heater_error";')
disp('constraints:')
disp('2 <= x1 <= 5')%Kh
disp('15 <= x2 <= 25')%theta t
disp('0 <= x3 <= 4')%theta d
disp('15 <= x4 <= 30')%T_env
%----------------------------------------------------------
clear all 
close all 
format compact

data_sys = load('airheater_logfile.txt');

%% Selecting the estimation and validation dataset
L=length(data_sys);
F=0.7;%First fraction (portion) of logfile to be used for model adaption:
N=floor(L*F); %N is number of samples used for estimation.
del_t=0.1;

%Rest used for visual validation.
t_estim=del_t*[1:N]';
u_estim=data_sys(1:N,2);
y_estim=data_sys(1:N,3);
t_valid=del_t*[N+1:L]';
u_valid=data_sys(N+1:L,2);
y_valid=data_sys(N+1:L,3);
t_total=del_t*[1:L]';
u_total=data_sys(1:L,2);
y_total=data_sys(1:L,3);

%% Grid Search Method
%Initialization:
x1_min=2;x1_max=5;N_x1=50;
x1_array=linspace(x1_min,x1_max,N_x1);

x2_min=15;x2_max=25;N_x2=50;
x2_array=linspace(x2_min,x2_max,N_x2);

x3_min=0;x3_max=4;N_x3=101;
x3_array=linspace(x3_min,x3_max,N_x3);

x4_min=15;x4_max=30;N_x4=50;
x4_array=linspace(x4_min,x4_max,N_x4);

f_min=inf;
x1_opt=-inf;
x2_opt=-inf;
x3_opt=-inf;
x4_opt=-inf;

tic;

for k_x1=1:length(x1_array)
    x1=x1_array(k_x1);

    for k_x2=1:length(x2_array)
        x2=x2_array(k_x2);
        
          for k_x3=1:length(x3_array)
          x3=x3_array(k_x3);
          
            for k_x4=1:length(x4_array)
            x4=x4_array(k_x4);

            %Objective function:
            f=AC_heater_error(x1,x2,x3,x4,u_estim, y_estim);

        %Improving the previous solution:
         if f <= f_min,
             f_min=f;
             x1_opt=x1;
             x2_opt=x2;
             x3_opt=x3;
             x4_opt=x4;
         end

            end
          end
    end
end

%--------------------------------------------------
%Displaying the optimal solution:

disp('-------------------')
disp('Optimal solution:')

f_min
K_opt=x1_opt
thetaT_opt=x2_opt
thetaD_opt=x3_opt
T_env=x4_opt

%--------------------------------------------------
%Calculation of execution times:
disp('-------------------')
disp('Execution times of running the nested for-loops:')

dt_elapsed_if=toc
dt_elapsed_per_cycle_if=toc/(N_x1*N_x2*N_x3*N_x4)
N_tot=N_x1*N_x2*N_x3*N_x4;


%% Model Validation
A=(1-del_t/thetaT_opt);
B=[del_t*K_opt/thetaT_opt del_t/thetaT_opt] ;
C=1;
D=[0 0];
model_est=ss(A,B,C,D,del_t,'InputDelay',round(thetaD_opt/del_t));
u_total=[u_total T_env.*ones(length(u_total),1)];
x0=y_total(1);
[y_sim,t,x] = lsim(model_est,u_total,[],x0);


%% Plots
h=figure; %Getting figure handle
fig_posleft=8;fig_posbottom=1.5;fig_width=24;fig_height=20;
fig_pos_size_1=[fig_posleft,fig_posbottom,fig_width,fig_height];

set(gcf,'Units','centimeters','Position',fig_pos_size_1);
figtext='Estimation of model params of Air-Heater';
set(gcf,'Name',figtext,'NumberTitle','on')

subplot(2,1,1)
figure(1)
plot(t_total,y_sim,'r',t_total,y_total,'b');
title('Real y (blue). Simulated y with adapted model (red). Green: Interval for adaption.')
grid minor
ylim([20,35]);
ylabel('[V]');xlabel('t [s]')

subplot(2,1,2)
plot(t_total,u_total,'b');
title('Applied control sigal, u, to both real and simulated process. Green: Interval for adaptation.')
grid minor
ylim([0,4]);
ylabel('[V]');xlabel('t [s]')


%% Function for calculating the error
function f = AC_heater_error(x1,x2,x3,x4,u,y)
Kh=x1;
theta_t=x2;
dt=0.1;
theta_d=max(0,round(x3/dt)-1);

Tprev_meas=y(1:end-1);
Tout_meas=y(2:end);
u = [u(1)*ones(theta_d,1); u(1:end-theta_d-1)];
T_env=x4.*ones(length(u),1);
T_est=((1-dt/theta_t).*Tprev_meas)+((Kh*dt/theta_t).*u)+(dt/theta_t.*T_env);
error=Tout_meas-T_est;
f=error'*error;
end

