%----------------------------------------------------------
%Parameter estimation of a AirHeater with n4sid subspace identification.
%Finn Haugen (finn.haugen@usn.no)
%----------------------------------------------------------

clear all
close all

%Loads data from file into workspace.
load airheater_logfile.txt;
Ts=0.1; %Sampling interval

%% Selecting the estimation and validation dataset
L=length(airheater_logfile);%(Matrix name becomes same as logfile name.)
F=0.8;%First fraction (portion) of logfile to be used for model adaption:
N=floor(L*F); %N is number of samples used for estimation.

%Rest used for visual validation.
t_estim=Ts*[1:N]';
u_estim=airheater_logfile(1:N,2);
y_estim=airheater_logfile(1:N,3);
t_valid=Ts*[N+1:L]';
u_valid=airheater_logfile(N+1:L,2);
y_valid=airheater_logfile(N+1:L,3);
t_total=Ts*[1:L]';
u_total=airheater_logfile(1:L,2);
y_total=airheater_logfile(1:L,3);

%% Grid Search method to identify the optimal value of control signal delay
x1_min=1;x1_max=40;N_x1=40;
x1_array=linspace(x1_min,x1_max,N_x1);

f_min=inf;
x1_opt=-inf;

for k_x1=1:length(x1_array)
    
    x1=x1_array(k_x1);
    % Subspace model identification
    modelorder=1;%Defines order of estimated model.
    % Estimation of model. Model is on internal theta-format:
    data_estim=iddata(y_estim,u_estim,Ts); %data used for estimation
    [model_est]=n4sid(data_estim,modelorder,'InputDelay',x1); 
    f=model_est.Report.Fit.MSE; %least square error
            if f <= f_min,
                 f_min=f;
                 x1_opt=x1;
            end
end 

data_total=iddata(y_total,u_total,Ts); % the complete dataset
[model_est,x0]=n4sid(data_total,modelorder,'InputDelay',x1_opt); % Generating the subspace model
[y_sim,t,x] = lsim(model_est,data_total.InputData,[],x0); %simulating the 
LSE=model_est.Report.Fit.MSE %least square error

%% Plots
h=figure; %Getting figure handle
fig_posleft=8;fig_posbottom=1.5;fig_width=24;fig_height=20;
fig_pos_size_1=[fig_posleft,fig_posbottom,fig_width,fig_height];

set(gcf,'Units','centimeters','Position',fig_pos_size_1);
figtext='Estimation of model params of DC motor';
set(gcf,'Name',figtext,'NumberTitle','on')

subplot(2,1,1)
figure(1)
plot(t_total,y_sim,'r',t_total,y_total,'b');
title('Real y (blue). Simulated y with adapted model (red). Green: Interval for adaption.')
grid minor
ylim([15,45]);
ylabel('[Temperature]');xlabel('t [s]')

subplot(2,1,2)
plot(t_total,u_total,'b');
title('Applied control sigal, u, to both real and simulated process. Green: Interval for adaptation.')
grid minor
ylim([0,4]);
ylabel('[V]');xlabel('t [s]')

